#!/usr/bin/python
# coding: utf-8

"""
A large part of the graphical interface comes from Nour Bouazza's master thesis (ULg).
Modifications : Simon Lienardy
From Nour's file, I added weight on the link between hosts in order to be able to construct fakeroute topologies.
The idea is to be able to import topologies and add manually weights (this mecanism could be automated) and construct the 
topologies.

All internals functions were rewrited to use networkx library.
"""

from faketopo import FakerouteFilesCreator, MerlinParser

import Tkinter as tk

import tkMessageBox as tkMess
import networkx as nx
import tkFileDialog as tkFDiag

import os
import gc
import collections
import itertools as it

DEFAULT_WEIGHT = 1
DEFAULT_DELAY = 0.0
DEFAULT_WEIGHT_NAME = "weight"		# Defaut name of the Weight in the graph.
DEFAULT_IS_WEIGHT_INVERSED = False	# Defaut behaviour = do not inverse weight while computing shortest path


class ASview(tk.Frame):
	"""
	
	"""

	def __init__( self, parent=None):

		# Main window
		tk.Frame.__init__(self, parent)
		
		# Info frame
		self.InfotopLevel = self.winfo_toplevel()
		self.InfotopLevel.title('ASview')
		self.height = 640
		self.width  = 800
		self.top = self.winfo_toplevel()

		# Window resizing
		self.columnconfigure(1, weight=1)
		self.rowconfigure(0, weight=1)
		self.pack(expand=True, fill='both')

		# Left_bar
		self.init_left_bar()
		
		# Menu bar
		self.init_menu_bar()

		# Drawing board
		self.frame = tk.Frame(self)
		
		# Board
		self.init_board()

		# Drawing board resizing
		self.frame.rowconfigure(0, weight=1)
		self.frame.columnconfigure(1, weight=1)
		self.frame.grid(row=0, column=0, sticky='NSEW')

		# Drawing board position
		self.frame.grid(column=1, row=0)

		# Nodes binds
		self.host_binds = tk.Label()
		self.host_binds.bind('<ButtonPress-1>', self.click_on_host)
		self.host_binds.bind('<B1-Motion>', self.drag_host)
		self.host_binds.bind('<ButtonRelease-1>', self.drop_host)
		self.host_binds.bind('<Button-2>', self.delete_host)

		# Internal structure
		self.init_internal()

		# Edge : select to delete
		self.selection = None
		self.bind('<KeyPress-Delete>', self.delete_link_selected)
		self.bind('<KeyPress-F1>', self.info_about)
		self.focus()

	def init_left_bar(self):
		"""
		Initialisation of the left side bar.
		"""
		self.left_bar = tk.Frame(self, bg='ivory3')
		self.left_bar.grid(column=0, row=0, sticky='NSEW')
		#Label
		label_1 = tk.Label(self.left_bar, text="", bg='ivory3')
		label_1.pack(fill='x')
		#Host button
		button_host = tk.Button(self.left_bar, text='Host', command=self.click_button_host)
		button_host.pack(fill='x')
		#Link button
		button_link = tk.Button(self.left_bar, text='Link', command=self.click_button_link)
		button_link.pack(fill='x')
		#Label
		label_2 = tk.Label(self.left_bar, text="", bg='ivory3')
		label_2.pack(fill='x')
		#Save GML button
		button_gen = tk.Button(self.left_bar, text='Save GML', command=self.click_button_write_gml)
		button_gen.pack(fill='x')
		#Open button
		button_gen = tk.Button(self.left_bar, text='Open GML', command=self.click_button_open_gml)
		button_gen.pack(fill='x')
		#Label
		label_3 = tk.Label(self.left_bar, text="", bg='ivory3')
		label_3.pack(fill='x')
		#Fakeroute button
		button_gen = tk.Button(self.left_bar, text="Fakeroute", command=self.click_button_fakeroute)
		button_gen.pack(fill='x')
		button_gen = tk.Button(self.left_bar, text="Import Merlin", command=self.click_button_merlin)
		button_gen.pack(fill='x')
		button_gen = tk.Button(self.left_bar, text="ECMP", command=self.click_button_ecmp)
		button_gen.pack(fill='x')
		#Label
		label_4 = tk.Label(self.left_bar, text="", bg='ivory3')
		label_4.pack(fill='x')
		#Refresh button
		button_gen = tk.Button(self.left_bar, text='Refresh', command=self.On_button_click_fresh)
		button_gen.pack(fill='x')
	
		self.str_label = tk.StringVar()
		self.str_label.set("")
	
		label = tk.Label(self.left_bar, textvariable=self.str_label, bg='ivory3', fg='red')
		label.pack(fill='x')

	def init_menu_bar(self):
		"""
		Initialisation of the menu bar.
		"""
		menu_bar = tk.Menu(self.top, bg='ivory3')
		self.top.configure(menu = menu_bar)

		#File menu
		file_menu = tk.Menu(menu_bar, tearoff=False)
		menu_bar.add_cascade(label="File", menu=file_menu)
		file_menu.add_command(label="New", command=self.On_button_click_fresh)
		file_menu.add_separator()
		file_menu.add_command(label="Open", command=self.click_button_open_gml)
		file_menu.add_command(label="Save as", command=self.click_button_write_gml)
		file_menu.add_separator()
		file_menu.add_command(label='Quit', command=self.quit)       

		#Edit menu
		edit_menu = tk.Menu(menu_bar, tearoff=False )
		menu_bar.add_cascade(label="Edit", menu=edit_menu )
		edit_menu.add_command(label="Host", command=self.click_button_host)
		edit_menu.add_command(label="Link", command=self.click_button_link)

		#About menu
		about_menu = tk.Menu(menu_bar, tearoff=False )
		menu_bar.add_cascade(label="About", menu=about_menu)
		about_menu.add_command(label="Information", command=self.On_button_click_about)

		#Configuration menu
		
		self.weight_name = DEFAULT_WEIGHT_NAME
		self.inverse_wgt = DEFAULT_IS_WEIGHT_INVERSED
	
		config_menu = tk.Menu(menu_bar, tearoff=False)
		menu_bar.add_cascade(label="Config", menu=config_menu)
		config_menu.add_command(label="Options", command=self.on_button_click_options)

	def quit(self):
		"""
		Quit the program
		"""
		tk.Frame.quit(self)

	def init_board(self):
		"""
		Initialisation of the drawing board (Tkinter.Canvas)
		"""
		self.board = tk.Canvas(self.frame, width=self.width, height=self.height, bg='white')
		self.board.grid( row=0, column=1, sticky='NSWE')
		#Scroll bars
		scrollHor = tk.Scrollbar(self.frame, orient='horizontal', command=self.board.xview)
		scrollVer = tk.Scrollbar(self.frame, orient='vertical', command=self.board.yview)
		scrollVer.grid(row=0, column=2, sticky='NS')
		scrollHor.grid(row=1, column=1, sticky='EW')
		self.board.configure(xscrollcommand=scrollHor.set, yscrollcommand=scrollVer.set)
		#Bindings
		self.board.bind('<ButtonPress-1>', self.click_on_board)

	def init_internal(self):
		"""
		Internal variables initialisation	
		"""

		#Default state is 'Host'
		self.current_state = 'Host'
		
		#Dicts mapping internals and grapical objects
		self.widget_to_item = {}
		self.item_to_widget = {}

		#Edges
		self.edge = self.edge_widget = None
		self.edge_x = self.edge_y = self.edge_item = None
		self.edges = {}
		self.source = None
		
		self.nb_to_widget = {} # from logical number to wigdet. Used fot import.

		# key = edge item number, value = tuple of the logical nodes id
		self.assoc_links = {}

		# key = node, val = item 
		self.dict_assoc = {}
		# key = edge, val = item
		self.dict_assoc_edge = {}

		#Internals
		self.nbr_edge = 0
		self.nbr_pc = 0 ## Deprecated -> use len(digraph) instead 

		# key = item (internal repr.), val = type of the node
		self.dict_type = {}
		
		#True graph representation:
		self.digraph = nx.DiGraph()

		self.largest_id = 0
		
		# Used to transfer properties
		self.edge_prop = EdgeProperty()
		
	def quit_state(self):
			if self.current_state == "Selected1":
				del self.path_first_node
				
			self.str_label.set("")
		
	######################################### left_bar and  menu bar fonctions
	
	def click_button_host(self):
		""" 
		Change state to Host
		"""
		self.quit_state()
		self.current_state = 'Host'

	def click_button_link(self):
		"""
		Change state to Link
		"""
		self.quit_state()
		self.current_state = 'Link'

	def click_button_write_gml(self):
		"""
		Handler to write the topology in .GML format
		"""
		
		path = tkFDiag.asksaveasfilename(title="Choose an emplacement for your GML file", defaultextension='.gml')

		if not path:
			return

		nx.write_gml(self.digraph, path)

	def click_button_open_gml(self):
		"""
		Handler to load a topology specified in .GML format
		"""
		gml_file = tkFDiag.askopenfilename(title="Choose a GML file", defaultextension='.gml')

		if gml_file != "":
			try:
				# Reset the display
				self.On_button_click_fresh()
				
				# GML parsing thanks to networkx capabilities
				read = nx.read_gml(gml_file)
				
				self.digraph = nx.DiGraph(read)

			except: 
				tkMess.showerror(title="Message", message="Please, be sure to use a correct GML file")
	
			self.print_topology('graphics', size_factor=10)
	
	def click_button_fakeroute(self):
		""" 
		Generate fakeroutes files
		@author: Simon Lienardy
		"""
		
		f = FakerouteFilesCreator(self.digraph)

		self.directory = tkFDiag.askdirectory(title="Choose an emplacement for your lab",)

		if not self.directory:
			return

		if  self.directory == () or self.directory == None :
			return
		
		name = self.find_name(self.directory)
		
		if not os.path.exists(self.directory):
			os.makedirs(self.directory)
		
		if self.directory != "":
			
			nx.write_gml(self.digraph, ''.join((self.directory, '/', name)))
			f.generate_files(directory=str(self.directory), inverse=self.inverse_wgt, weight_name=self.weight_name)
					
			tkMess.showinfo(title="Message", message="Files generated")
		else : 
			tkMess.showinfo(title="Message", message="Please, check your topology.")
		
	def click_button_merlin(self):
		"""
		Load a Merln topology and display it (erase the current state). 
		Warning : nodes must go from 0 to #nodes-1
		"""
		
		self.On_button_click_fresh()# Will erase all...
		gc.collect() # if data are huge after re-init...
		
		nodes_file = tkFDiag.askopenfilename(title="Choose a Nodes file", defaultextension='.nodes')
		links_file = tkFDiag.askopenfilename(title="Choose a Links file", defaultextension='.links')
		
		if not (nodes_file and links_file):
			return 
		
		mp = MerlinParser()
		
		(node_list, edge_list) = mp.get_and_parse(nodes_file, links_file)

		build_graph = nx.Graph()
		
		build_graph.add_nodes_from(node_list)
		build_graph.add_edges_from(edge_list)

		self.digraph = nx.DiGraph(build_graph)
		
		# Here, the attrbute 'graphics' cannot exists so passing it is useless...
		self.print_topology('graphics')
		
	def click_button_ecmp(self):
		"""
		Set the state to waiting nodes
		"""	
		self.quit_state()
		self.current_state = 'Selected0'
		
		self.str_label.set("Select a host")
	
	
	def print_topology(self, attribute, size_factor=2):
		"""
		@param attribute : element in GML file that describe item positions (must be a str)
		@param size_factor : enable to control the space between displayed items
		@note: the nodes are not needed : they are stored in the Graph.
		"""
		
		# It is useless to compute layout if the information is already present in the graph.
		if self.has_node_attribute(attribute):
			
			# This will "center" the graph while displaying...
			# Select the nodes as a list, take the first node (that is a tuple) and take the second member of the tuple to get node data
			x_offset = self.digraph.nodes(True)[0][1]['graphics']['center']['x']
			y_offset = self.digraph.nodes(True)[0][1]['graphics']['center']['y']
			
			
			layout = {node : (node_data['graphics']['center']['x'] - x_offset ,node_data['graphics']['center']['y'] - y_offset)
					 for (node, node_data) in self.digraph.nodes_iter(True)}
		else:
			layout = nx.graphviz_layout(self.digraph)
		
		sized_layout = {k:tuple(x * size_factor for x in v) for (k,v) in layout.iteritems()}
		
		
		display_host = self.display_host
		display_link = self.display_link
		
		for (k,v) in sized_layout.iteritems():
			display_host(k, v[0], v[1], 'router') 
		
		for (src, dest) in self.digraph.to_undirected().edges():
			display_link(src, dest, sized_layout)
		
		del self.nb_to_widget
		
	def has_node_attribute(self, attribute):
		"""
		@return True if all the nodes of self.digraph has the attribute 'attribute'.
		"""
		result_list = [attribute in node_data for (_, node_data) in self.digraph.nodes_iter(True)]
		return result_list.count(False) == 0

	def On_button_click_fresh(self):
		"""
		Re-init the program
		"""
		#You should not re-use code from someone else...
		tmp_dico = {}
		i = 0
		for k,_ in self.item_to_widget.iteritems():
			tmp_dico[i] = k
			i += 1
		for j in range(0, i):
			self.delete_item(tmp_dico[j])
		
		# Init functions call
		self.init_internal()
		self.init_board()
		self.focus()
	
	def find_name(self, directory):
		"""
		Get a directory name and add ".gml" to it
		 - directory : path of the directory
		"""
		if directory == None:
			return
		name = directory.split('/')
		return name[len(name) - 1]+".gml"

	# About window
	def On_button_click_about(self):
		tkMess.showinfo(title="Information", message="""This GUI helps to draw topology connecting nodes with (weighted) edges and launch several tools on it"""
					"""\n\nAuthor : LIENARDY Simon\nContact: simon.lienardy@ulg.ac.be\nVERSION 0.1""")
	def info_about(self, event):
		tkMess.showinfo(title="Information", message="""This GUI helps to draw topology connecting nodes with (weighted) edges and launch several tools on it"""
					"""\n\nAuthor : LIENARDY Simon\nContact: simon.lienardy@ulg.ac.be\nVERSION 0.1""")
		
	# Configuration window
	def on_button_click_options(self):
		
		pop = popup_window_options(None, self.weight_name, self.inverse_wgt)
		self.master.wait_window(pop.top)
		
		self.weight_name = pop.attribute_string
		self.inverse_wgt = pop.inverse
		
	#######################################################################Board

	def canvasx( self, x_root ):
		"""
		X coordinate in the board
		"""
		c = self.board
		return c.canvasx(x_root) - c.winfo_rootx()
	
	def canvasy( self, y_root ):
		"""
		Y coordinate in the board
		"""
		c = self.board
		return c.canvasy( y_root ) - c.winfo_rooty()

	def select_item(self, item):
		"""
		Graphical item selection
		"""
		self.selection = item

	########################################################################node
	
	def display_host(self, id_pc, coord_x, coord_y, state):
		"""
		Display an host on the canvas.
		"""

		if self.largest_id <= id_pc:
			self.largest_id = id_pc + 1

		# Check the state
		if state == 'middlebox':
			state = 'mb'
		elif state == 'router':
			state = 'Host'
		else:
			state = 'Host'
		
		x = coord_x
		y = coord_y

		#Displayed name
		name = state+" "+ str(id_pc) + "i"

		#Graphicaly, a host is a button (add an image?)
		image = tk.Button( self.board, text=name, compound='top')
		#binds
		bindtags = [str(self.host_binds)]
		image.bindtags(tuple(bindtags))
		item = self.board.create_window(x, y, anchor='c', window=image, tags='Host')
		
		#save dict type
		self.dict_type[item] = state

		#Modify internals
		self.widget_to_item[image] = item
		self.item_to_widget[item] = image
		self.select_item(item)
		image.links = {}
		
		self.nb_to_widget[id_pc] = image
		
		self.dict_assoc[item] = id_pc

		
	def display_link(self, src, dest, layout):
		"""
		Fonction servant surtout a remplir la board avec un fichier GML charge
		Ajout d'un lien
		@param src: Logical id of source node
		@param dest: Logical id of dest node 
		@param layout: Dict (k,v) = (node id, tuple of x, y positions) of the nodes 
		"""
		
		# 1) Draw the edge
		self.edge = self.board.create_line(layout[src][0], layout[src][1], layout[dest][0], layout[dest][1], width=2, fill='black', tag='link')
		
		arrowshape = (40,40,15)
		link = self.edge
		
		if not self.digraph.has_edge(src, dest):
			self.board.itemconfig(link, arrow=tk.FIRST, arrowshape=arrowshape)
			
		if not self.digraph.has_edge(dest, src):
			self.board.itemconfig(link, arrow=tk.LAST, arrowshape=arrowshape)
		
		# add bidings.
		self.link_bindings()
		
		# 2) Fill the data structures
		widget_src = self.nb_to_widget[src]
		widget_dest = self.nb_to_widget[dest]
		
		self.add_link(widget_src, widget_dest)
		
		self.assoc_links[link] = (src, dest)
		

	def click_on_board(self, event):
		"""
		Left click on board handler
		Add an host on the board.
		"""
		
		board = self.board
		x = board.canvasx(event.x)
		y = board.canvasy(event.y)

		#depends on current state
		if self.current_state == 'Host':
			name = "host " + str(self.largest_id)
		elif self.current_state == 'Link':
			return
		
		# Add the new node in the graph with the proper state
		self.digraph.add_node(self.largest_id, state=self.current_state)

		#Graphicaly, it's a button. Change for an image ?
		image = tk.Button( self.board, text=name, compound='top' )
		#binds
		bindtags = [str(self.host_binds)]
		image.bindtags(tuple(bindtags))

		#Localisation of the button
		item = self.board.create_window(x, y, anchor='c', window=image, tags='Host')

		#Save item type
		self.dict_type[item] = self.current_state

		#Fill the internal data structures
		self.widget_to_item[image] = item
		self.item_to_widget[item] = image
		self.select_item(item)
		image.links = {}
		self.dict_assoc[item] = self.largest_id


		self.largest_id += 1

	def click_on_host( self, event ):
		"""
		Click on host handler
		If Host state -> select the host
		If Link state -> draw an edge
		"""
		if self.current_state is 'Link':
			self.start_link(event)
		elif self.current_state is "Selected0":
			self.str_label.set("Select a snd host")
			self.path_first_node = event.widget
			self.current_state = "Selected1"
			
		elif self.current_state is "Selected1":	
			self.all_paths_between(self.path_first_node, event.widget)
			self.quit_state()
			self.current_state = 'Host'
		else:
			self.select_host(event)
		return 'break'

	def drag_host( self, event ):
		"""
		Drag an host handler
		If Host state : Host movement
		IF Link state : Edge building
		"""
		if self.current_state is 'Link':
			self.drag_link(event)
		else:
			self.drag_host_around(event)

	def drop_host( self, event ):
		"""
		Drop on host
		If Host state : Host movement end
		If Link state : Edge drawing end
		"""
		if self.current_state is 'Link':
			self.finish_link(event)

	def drag_host_around(self, event):
		"""
		Deplace a Host on the board.
		"""
		board = self.board
		x = self.canvasx(event.x_root)
		y = self.canvasy(event.y_root)
		w = event.widget

		# Sanity check that avoid perfect overlapping of two nodes that would cause edge
		# tracking more difficult
		linked_host_item = (self.widget_to_item[dest] for (dest, _) in w.links.iteritems())
		linked_host_position  = (tuple(board.coords(item)) for item in linked_host_item)
		if (x,y) in linked_host_position:
			return

		item = self.widget_to_item[w]
		# Displacement of the Node :
		board.coords(item, x, y)
		
		# Edge tracking. If an edge has an arrow, the arrow must stay on the same side of the edge.
		# Let's create a generator of tuple : (link id, link coord, host coord)
		w_t_i = self.widget_to_item  # Speed up the lookup. 
		update_generator = ((link, board.coords(link), board.coords(w_t_i[dest])) for dest, link in w.links.iteritems())
		
		# Build the 4-uple to update line position with board.coords
		# The if-condition ensure that the fixed side of the edge is not inversed with the moving one 
		# resulting in the inversion of the edge that would be bad if the line carries an arrow.
		update_action = ((board.coords(link, ncoord[0], ncoord[1], x,y) ) if lcoord[0:2] == ncoord 
						else  
						 (board.coords(link, x, y,ncoord[0], ncoord[1])) for link, lcoord, ncoord in update_generator)
		
		# Consumption actually commit the changes.
		consume(update_action)
		

	def select_host(self, event):
		"""
		Host selection
		"""
		item = self.widget_to_item.get(event.widget, None)
		self.select_item(item)

	def delete_host(self, event):
		"""
		Host deletion
		"""
		item = self.widget_to_item.get(event.widget, None)
		widget = self.item_to_widget[item]
		
		tmp = self.dict_assoc[item]
		
		# Logical removal :
		self.digraph.remove_node(tmp) # Supposed to delete incident edges as well.

		#delete les liens qui le relient aussi
		for link in widget.links.values():
			self.delete_item_host(link)
			
		#del the graphical representation and remove it form dico's
		del self.item_to_widget[item]
		del self.widget_to_item[widget]
		self.board.delete(item)
	
	def delete_item(self, item):
		"""	
		Delete an item + graphical objet
		"""
		#Get the graphical widget
		widget = self.item_to_widget[item]	

		#delete incident edges
		for link in widget.links.values():
			self.delete_item_host(link)
		del self.item_to_widget[item]
		del self.widget_to_item[widget]
		self.board.delete(item)

	def delete_item_host(self,item):
		"""
		Delete incident edges to a host
		"""
		if item in self.edges:
			self.delete_link(item)
		self.board.delete(item)

	########################################################################Link
	def start_link(self, event):
		"""
		Begin the creation of a link
		"""
		#click near a host (but not on the host)
		if event.widget not in self.widget_to_item: 
			return

		#Src host coordinates
		w = event.widget
		item = self.widget_to_item[w]
		x, y = self.board.coords(item)
		#Find the item
		target = item
		
		if target in self.dict_assoc:	
			#Src node found from dicts.
			tmp = self.dict_assoc[target]
			self.source = tmp

		#Information about created edge
		self.edge = self.board.create_line(x, y, x, y, width=2, fill='black', tag='link')
		self.edge_x, self.edge_y = x, y
		self.edge_widget = w
		self.edge_item = item

	def drag_link(self, event):
		"""
		Edge movement. Source is fixed, the dest is the mouse pointer
		"""
		if self.edge is None:
			return

		x = self.canvasx(event.x_root)
		y = self.canvasy(event.y_root)
		board = self.board
		board.coords(self.edge, self.edge_x, self.edge_y, x, y)

	def release_link(self, _event):
		"""
		release the edge
		"""
		#sur rien : abandon du lien
		if self.edge is not None:
			self.board.delete(self.edge)
		self.edge_widget = self.edge_item = self.edge = None

	def finish_link(self, event):
		"""
		Get the dest Host of an edge and finish edge creation.
		"""
		if self.edge is None:
			return

		source = self.edge_widget
		board = self.board
		
		#get the destination
		x, y = self.canvasx(event.x_root), self.canvasy(event.y_root) # coord transform.
		target = self.find_item(x, y)
		dest = self.item_to_widget.get(target, None)

		#if destination is not correct, the edge is not created
		if (source is None or dest is None or source == dest or dest in source.links or source in dest.links):
			self.release_link(event)
			return

		#Fix the destination of the edge.
		x, y = board.coords( target ) # center of the target
		board.coords( self.edge, self.edge_x, self.edge_y, x, y) # placement of the end of the graphical edge
		self.link_bindings()
		self.add_link(source, dest)

		# New edge -> fill internals
		self.nbr_edge += 1
		if target in self.dict_assoc:
			tmp = self.dict_assoc[target]
			# By default, links are bidirectionals
			self.digraph.add_edge(self.source, tmp, {self.weight_name : DEFAULT_WEIGHT})
			self.digraph.add_edge(tmp, self.source, {self.weight_name : DEFAULT_WEIGHT})
			#try :
			self.assoc_links[self.edge] = (self.source, tmp)
		
		self.edge = self.edge_widget = None

	def find_item(self, x, y):
		"""	
		Search for an item located in (x,y) on the board
		"""
		items = self.board.find_overlapping(x, y, x, y)
		if len(items) == 0:
			return None
		else:
			return items[0]

	def add_link(self, source, dest):
		"""
		Add a linf to the model
		 - edge src
		 - edge dest
		"""
		source.links[dest] = self.edge
		dest.links[source] = self.edge
		self.edges[self.edge] = (source, dest)
		self.dict_assoc_edge[self.edge] = self.nbr_edge + 1

	def delete_link(self, link):
		"""
		Delete a link from the model
		"""
		pair = self.edges.get(link, None)
		if pair is not None:
			source, dest = pair
			del source.links[dest]
			del dest.links[source]

		if link is not None:
			del self.edges[link]
			del self.assoc_links[link]

	def delete_link_selected(self, _event):
		"""
		delete the selected link
		"""
		if self.selection == None:
			return

		item = self.selection
		#only suppress element if the item is an edge
		if item in self.edges:
			
			(s, r) = self.assoc_links[item]
			
			self.digraph.remove_edges_from([(s,r), (r,s)]) # must be a list of tuple...
			self.delete_link(item) # Attention, does not modify graph anymore.
			self.selection = None
			self.board.delete(item)

	def link_bindings( self ):
		"""
		Bindings for the link (edges)
		"""	
		def select(_event, link=self.edge):
			"""
			left click is selection
			"""
			self.select_item(link)

		def on_right_click(_event, link=self.edge):
			"""
			right click
			"""
			
			src_name = " ".join(self.edges[link][0].config()['text'][-1])
			dst_name = " ".join(self.edges[link][1].config()['text'][-1])
			
			self.update_edge_properties(link)
			
			self.w=popup_window_edge(None, self.edge_prop, self.weight_name, src_name, dst_name)
			self.master.wait_window(self.w.top)

			self.update_logical_edge(link)
			
			weight_sd = self.edge_prop.weight_sd
			weight_ds = self.edge_prop.weight_ds
			
			# Add graphical arrow shape to unidirectionnal edges
			arrowshape = (40,40,15)
			
			if not weight_sd and not weight_ds:
				self.delete_link(link)
				self.selection = None
				self.board.delete(link)
			else:
				if not weight_sd:
					self.board.itemconfig(link, arrow=tk.FIRST, arrowshape=arrowshape)
				elif not weight_ds:
					self.board.itemconfig(link, arrow=tk.LAST, arrowshape=arrowshape)
			
		def on_enter(_event, link=self.edge):
			"""
			Edge coloration if overlapped by the mouse 
			"""
			self.select_item(link)
			self.board.itemconfig(link, fill='red')

		def on_leave(_event, link=self.edge):
			"""
			Edge coloration end if no more overlapped	
			"""
			self.board.itemconfig(link, fill='black')
		
		# bindings
		self.board.tag_bind(self.edge, '<ButtonPress-1>', select)
		self.board.tag_bind(self.edge, '<ButtonPress-3>', on_right_click)
		self.board.tag_bind(self.edge, '<Enter>', on_enter)
		self.board.tag_bind(self.edge, '<Leave>', on_leave)

	def all_paths_between(self, src, dest):
		"""
		@param src: the source node's widget
		@param dest : the dest nodes's widget
		"""
		src_lg = self.dict_assoc[self.widget_to_item[src]]
		dest_lg = self.dict_assoc[self.widget_to_item[dest]]
		
		sh_path_lgth = nx.shortest_path_length(self.digraph, src_lg, dest_lg)
		
		# All paths between two nodes with a threshold of max_length = 1.5*shortest
		gen_path = nx.all_simple_paths(self.digraph, src_lg, dest_lg, int(sh_path_lgth * 1.5))
		
		#Coloration des paths
		
		# First, we inverse the dictionnary of the edges :
		invert_edges = {v : k for k,v in self.assoc_links.iteritems()}
		
		for path in gen_path:

			for i in range(0, len(path)-1):
				
				key = (path[i], path[i+1])
				rkey = (path[i+1], path[i])
				
				edge = invert_edges[key] if key in invert_edges else invert_edges[rkey]
		
				self.board.itemconfig(edge, fill='green')
				
	def update_edge_properties(self, link):
		"""
		"""
		
		(src, dst) = self.assoc_links[link]
		
		data_sd = self.digraph.get_edge_data(src, dst)
		data_ds = self.digraph.get_edge_data(dst, src)

		self.edge_prop.weight_sd = data_sd.get(self.weight_name, DEFAULT_WEIGHT) if data_sd else None
		self.edge_prop.weight_ds = data_ds.get(self.weight_name, DEFAULT_WEIGHT) if data_ds else None
		
	def update_logical_edge(self, link):
		"""
		"""
		
		(src, dst) = self.assoc_links[link]
		
		weight_sd = self.edge_prop.weight_sd
		weight_ds = self.edge_prop.weight_ds
		
		if weight_sd:
			if self.digraph.has_edge(src, dst):
				self.digraph.edge[src][dst][self.weight_name] = weight_sd
			else:
				self.digraph.add_edge(src, dst, {self.weight_name : weight_sd})
		else:
			self.digraph.remove_edges_from([(src, dst)]) # Die silently if does not exist
			
		if weight_ds:
			if self.digraph.has_edge(dst, src):
				self.digraph.edge[dst][src][self.weight_name] = weight_ds
			else:
				self.digraph.add_edge( dst, src,{self.weight_name : weight_ds})
		else:
			self.digraph.remove_edges_from([(dst, src)]) # Die silently if does not exist

###############################################################fenetre de config


class popup_window_edge():
	"""
	Edge options class
	"""

	def __init__(self,master, properties, weight_name, src_name, dst_name):
		
		self.edge_prop = properties
		
		self.top = tk.Toplevel(master)
		top = self.top
		self.title = tk.Label(top,text="Modifying an edge", bg = "white", fg = "red", font = "Times", relief=tk.RAISED).pack()
		
		# Added by S. Lienardy
		text_sd = ' '.join((weight_name, src_name, "->", dst_name))
		text_ds = ' '.join((weight_name, dst_name, "->", src_name))
		
		self.set_weight_sd_l = tk.Label(top, text=text_sd)
		self.set_weight_sd_l.pack()
		
		weight_sd = self.edge_prop.weight_sd
		weight_ds = self.edge_prop.weight_ds
		
		text_value_sd = str(weight_sd) if weight_sd else ''
		text_value_ds = str(weight_ds) if weight_ds else ''
		
		self.set_weight_sd_e = tk.Entry(top)
		self.set_weight_sd_e.insert(0, text_value_sd)
		self.set_weight_sd_e.pack()
		
		self.set_weight_ds_l = tk.Label(top, text=text_ds)
		self.set_weight_ds_l.pack()
		
		self.set_weight_ds_e = tk.Entry(top)
		self.set_weight_ds_e.insert(0, text_value_ds)
		self.set_weight_ds_e.pack()
		
		# End addition 

		self.b = tk.Button(top,text='Ok',command=self.ok)
		self.b.pack()

	def ok(self):
		weight_sd = self.set_weight_sd_e.get()
		weight_ds = self.set_weight_ds_e.get()
		
		self.edge_prop.weight_sd = int(weight_sd) if weight_sd else None
		self.edge_prop.weight_ds = int(weight_ds) if weight_ds else None
		self.top.destroy()


class EdgeProperty():
	
	def __init__(self):
		self.weight_sd = DEFAULT_WEIGHT
		self.weight_ds = DEFAULT_WEIGHT


class popup_fakeroute():
	"""
	Class defining a popup window used for fakeroute options.
	"""
	def __init__(self, master, potential_sources):
		
		self.top = tk.Toplevel(master)
		top = self.top
		self.title=tk.Label(top,text=" Sources definitions ", font = "Times").pack()
		
		self.nb_nodes_l = tk.Label(top,text="".join(("Number of potential sources: ", str(potential_sources))))
		self.nb_nodes_l.pack()
	
		self.nb_start_source_l = tk.Label(top,text="Number of source at start: ")
		self.nb_start_source_l.pack()
		
		self.nb_start_source_e = tk.Entry(top)
		self.nb_start_source_e.pack()
		
		self.nb_step_source_l = tk.Label(top,text="Step of sources number: ")
		self.nb_step_source_l.pack()
		
		self.nb_step_source_e = tk.Entry(top)
		self.nb_step_source_e.pack()
		
		self.nb_end_source_l = tk.Label(top,text="Number of sources at end: ")
		self.nb_end_source_l.pack()
		
		self.nb_end_source_e = tk.Entry(top)
		self.nb_end_source_e.pack()
		
		self.b=tk.Button(top,text='Ok',command=self.ok)
		self.b.pack()
		
	def ok(self):

		nb_start = self.nb_start_source_e.get()
		nb_step = self.nb_step_source_e.get()
		nb_end = self.nb_end_source_e.get()
		
		self.params = ( int(nb_start) if nb_start else 0,
						int(nb_step) if nb_step else 0,
						int(nb_end) + 1 if nb_end else 0)
		
		self.top.destroy()
	

class popup_window_options():
	"""
	Class defining a popup window used to manage options.
	"""
	
	def __init__(self, master, weight_name, inverse_wgt):
		
		self.attribute_string = weight_name
		self.var = tk.IntVar()
		self.inverse = inverse_wgt
		
		if self.inverse:
			self.var.set(1)
		else:
			self.var.set(0)
		
		self.top=tk.Toplevel(master)
		top = self.top
		self.title=tk.Label(top,text="Options", font = "Times").pack()

		self.set_name_l = tk.Label(top,text="Name of the weight attribute: ")
		self.set_name_l.pack()

		self.set_name_e = tk.Entry(top)
		self.set_name_e.insert(0, self.attribute_string)
		self.set_name_e.pack()
		
		check_invert = tk.Checkbutton(top, text="Inverse to compute weight", variable=self.var)
		check_invert.pack()

		self.b=tk.Button(top,text='Ok',command=self.ok)
		self.b.pack()
			
	def ok(self):
		self.attribute_string = self.set_name_e.get()
		self.inverse = bool(self.var.get())
		
		self.top.destroy()
	
####################################################################################################


def consume(iterator, n=None):
	"""
	This comes from itertools recipes
	
	Advance the iterator n-steps ahead. If n is none, consume entirely.
	"""
	# Use functions that consume iterators at C speed.
	if n is None:
		# feed the entire iterator into a zero-length deque
		collections.deque(iterator, maxlen=0)
	else:
		# advance to the empty slice starting at position n
		next(it.islice(iterator, n, n), None)

if __name__ == '__main__':
	app = ASview(None)
	app.mainloop()
