#!/usr/bin/python
# coding: utf-8
'''
Created on 25 sept. 2014

@author: slienardy
'''

import sys
import os
import shutil
import errno
import fileinput
import cPickle as pk
import random as rd
import Constants as const

from PreprocessTraces import edges_to_paths


def get_available_sources(source_dir_path):
    """
    @param dir_path: directory where dir results can be found (as mapping, etc.)
    """
    return os.listdir(source_dir_path)
    
def get_mapping(mapping_file_path):
    """ 
    Extract the mapping file
    """
    with open(mapping_file_path, 'r') as mapping_file:
            ip_to_id = pk.load(mapping_file)
            
    return ip_to_id

def sample(sources, nb_sources, nb_samples):
    """
    """
    
    return [rd.sample(sources, nb_sources) for _ in xrange(0, nb_samples, 1)]

def preprocess_traces(raw_traces_dir, preprocessed_dir,  mapping_dict):
    """
    """
    
    
    # Easier to ask for forgiveness than permission
    try:
        os.makedirs(preprocessed_dir)
    except OSError, e:
        # The fact that the dir already exist does not matter
        if (e.errno != errno.EEXIST):
            raise e
            
    list_sources = os.listdir(raw_traces_dir)
    
    for source in list_sources:
        with open("".join((preprocessed_dir, "/", source)), 'w') as out_file:
            with open("".join((raw_traces_dir, "/", source)), 'r') as in_file:
                """
                The source is a string that contain the IP of the source and that is used as filename
                """
                edges_to_paths(in_file, out_file, mapping_dict, source)

def merge_traces(dir_path, to_merge, result_path):
    """
    """
    
    paths_to_merge = ("".join((dir_path, "/", name)) for name in to_merge)
    
    with open(result_path, 'w') as out_file:
        for line in fileinput.input(paths_to_merge):
            out_file.write(line)
    

def make_cplexable_dir_for_n_sources(nb_sources_dir, preprocess_dir, topo_dir, nb_sources, nb_samples, available_sources):
    """
    """
    samples = sample(available_sources, nb_sources, nb_samples)
    
    try:
        os.makedirs(nb_sources_dir)
    except OSError, e:
        # The fact that the dir already exist does not matter
        if (e.errno != errno.EEXIST):
            raise e
        
    sample_id = 1
        
    with open("".join((nb_sources_dir, "/", const.SOURCES_FILES_NAME)), "w") as sources_files:
    
        for sources in samples:
            sample_dir = "".join((nb_sources_dir, "/sources", str(sample_id)))
            
            sources_files.write(str(sample_id) + " " + " ".join(sources) + "\n")
            sample_id += 1
            
            make_cplexable_dir(sample_dir, topo_dir, preprocess_dir, sources)
    

def make_cplexable_dir(dir_path, topo_dir, preprocess_dir, sources_list):
    """
    """
    
    # create a directory:
    try:
        os.mkdir(dir_path)
    except OSError, e:
        # The fact that the dir already exist does not matter
        if (e.errno != errno.EEXIST):
            raise e
        
    # Copy of the files needed for cplex solver:
    # delete the "dump" and ".router" files:
    open("".join((dir_path, "/", const.RAW_TRACE_FILE)), 'a').close()
    open("".join((dir_path, "/", const.GRAPH_DUMP_FILE)), 'a').close()

    shutil.copy("".join((topo_dir, "/", const.GRAPH_GV_FILE)), dir_path)
    
    # Merge traces files into one named after what solver expect
    merge_traces(preprocess_dir, sources_list, "".join((dir_path, "/", const.TRACE_FILE)))

def main(args):
    """
    arg 1: path to the directory containing the topology
    arg 2: Number of sources to be taken into account
    arg 3: Nber of combinationss (nber of resulting combination that take nb_source into account)
    """
    
    # 1 get the arguments
    
    dir_path = args[1]
    nb_sources = int(args[2])
    
    try:
        nb_samples = int(args[3])
    except IndexError:
        nb_samples = const.NB_SAMPLES
    
    
    raw_traces_dir = "".join((dir_path, "/", const.RESULT_DIRECTORY))
    available_sources = get_available_sources(raw_traces_dir)
    
    mapping_file_path = "".join((dir_path, "/", const.MAPPING_FILE))
    mapping = get_mapping(mapping_file_path)
    
    preprocess_dir =  "".join((dir_path, "/", const.PREPROCESS_DIRECTORY))
    preprocess_traces(raw_traces_dir, preprocess_dir, mapping)
    
    nb_sources_dir = "".join((dir_path, "/", const.CPLEX_DIRECTORY, "/", str(nb_sources), "_" ,const.SOURCES))
    make_cplexable_dir_for_n_sources(nb_sources_dir, preprocess_dir, dir_path, nb_sources, nb_samples, available_sources)


if __name__ == '__main__':
    main(sys.argv)