#!/bin/bash

# Executables
EXE_EXTRACT="python scripts/exportGraph.py"
EXE_SOLVER="release/solver"

# Files
FILE_DMPTOPO="graph.dump"
FILE_GVTOPO="graph.gv"
FILE_TRACES="scamper_traces.largest_cmp.router.log"
FILE_PARSED_TRACES="traces.log"
FILE_RESULT="weights.log"
FILE_LOG="solver.log"

# Parameters
BASEDIR="$1"


# Check arguments
if [ $# -lt 1 ]; then
	echo "Error: insufficient arguments."
	exit 1
fi

if [ ! -d $BASEDIR ]; then
	echo "Error: invalid argument."
	exit 1
fi

if [ ! -f $BASEDIR/$FILE_DMPTOPO ]; then
	echo "Error: topology file not found."
	exit 1
fi

if [ ! -f $BASEDIR/$FILE_TRACES ]; then
	echo "Error: traces file not found."
	exit 1
fi


# Extract topology
if [ ! -f $BASEDIR/$FILE_GVTOPO ]; then
	echo -ne "Extracting topology...\t"
	$EXE_EXTRACT $BASEDIR/$FILE_DMPTOPO $BASEDIR/$FILE_GVTOPO &> $BASEDIR/$FILE_LOG
	if [ $? -eq 1 ]; then
		echo "[ERROR]"
		cat $BASEDIR/$FILE_LOG
		exit 1
	fi
	echo "[DONE]"
fi

# Parse traces
if [ ! -f $BASEDIR/$FILE_PARSED_TRACES ]; then
	echo -ne "Parsing traces...\t"
	sed -e 's/^ *//g' \
		-e 's/|[0-9.*]\+[^|]*//g' \
		-e 's/|ID[0-9]\+\( -> [0-9.*]\+\)\+//g' \
		-e '/^$/d' $BASEDIR/$FILE_TRACES > $BASEDIR/$FILE_PARSED_TRACES 2>> $BASEDIR/$FILE_LOG
	if [ $? -eq 1 ]; then
		echo "[ERROR]"
		cat $BASEDIR/$FILE_LOG
		exit 1
	fi
	echo "[DONE]"
fi

# Solve problem
echo -ne "Solving problem...\t"
time ( $EXE_SOLVER $BASEDIR/$FILE_GVTOPO $BASEDIR/$FILE_PARSED_TRACES -o $BASEDIR/$FILE_RESULT &>> $BASEDIR/$FILE_LOG )
if [ $? -eq 1 ]; then
	echo "[ERROR]"
	cat $BASEDIR/$FILE_LOG
	exit 1
fi
echo "[DONE]"

exit 0

