/**
 * @file	base.h
 *
 * @author	Francois Clad
 * @date	2012-03-12
 * @version	1.0
 *
 * @brief Basic macros for C programs
 */ 

#ifndef _BASE_H_
#define _BASE_H_

#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

#define INF UINT_MAX

#ifdef WARNING
#define PRINT_WARNING(...) fprintf(stderr, "[\033[0;33mWARNING\033[0;m] " __VA_ARGS__)
#else
#define PRINT_WARNING(...)
#endif /* WARNING */

#ifdef INFO
#define PRINT_INFO(...) fprintf(stderr, "[\033[0;34mINFO\033[0;m] " __VA_ARGS__)
#define PRINT_INFO_PLUS(...) fprintf(stderr, __VA_ARGS__)
#else
#define PRINT_INFO(...)
#define PRINT_INFO_PLUS(...)
#endif /* INFO */

#ifdef DEBUG
#define PRINT_DEBUG(...) fprintf(stderr, "[\033[0;32mDEBUG\033[0;m] " __VA_ARGS__)
#define PRINT_DEBUG_PLUS(...) fprintf(stderr, __VA_ARGS__)
#else
#define PRINT_DEBUG(...)
#define PRINT_DEBUG_PLUS(...)
#endif /* DEBUG */

#define PRINT_ERROR(format, ...) fprintf(stderr, "%s: " format, __FILE__, __VA_ARGS__)

#define errExit(msg) do { fprintf(stderr, "[\033[0;31mERROR\033[0;m] %s:%d: " msg ": %s\n", __FILE__, __LINE__, strerror(errno)); exit(EXIT_FAILURE); } while (0)

#define MIN(a,b) ((a < b) ? a : b)
#define MAX(a,b) ((a > b) ? a : b)

#define FALSE 0
#define TRUE 1

typedef char bool;


#endif /* _BASE_H_ */
