#include "hashtable.h"

static const unsigned int primes[] = { 53, 97, 193, 389, 769, 1543, 3079, 6151 };

hashtable* create_table (unsigned int size)
{
	hashtable* htab = NULL;
	
	if (size > 7)
		return NULL;		// Size not allowed
	
	if ((htab = (hashtable *)malloc(sizeof(hashtable))) == NULL)
		return NULL;		// Malloc error
	
	htab->length = primes[size];
	if ((htab->table = (entry **)calloc(htab->length, sizeof(entry*))) == NULL)
		return NULL;		// Calloc error
	
	htab->counter = 0;
	
	return htab;
}

int insert_value (hashtable* htab, char* key, uint value)
{
	uint ind = 0;
	entry* e = NULL;
	
	ind = hash(key) % htab->length;
	
	e = (entry *)malloc(sizeof(entry));
	if (e == NULL)
		return -1;			// Malloc error
	
	e->key = key;
	e->value = value;
	e->next = htab->table[ind];
	
	htab->table[ind] = e;
	htab->counter++;
	
	return 0;
}

uint search_value (hashtable* htab, char* key)
{
	entry* e = NULL;
	
	e = htab->table[hash(key) % htab->length];
	
	while (e != NULL)
	{
		if (strcmp(key,e->key) == 0)
			return e->value;
		e = e->next;
	}
	
	return -1;	// Not found !
}

uint remove_value (hashtable* htab, char* key)
{
	uint ind = 0,
		 val = 0;
	entry *e = NULL, 
		  *last = NULL;
	
	ind = hash(key) % htab->length;
	
	e = htab->table[ind];
	
	while (e != NULL)
	{
		if (strcmp(key,e->key) == 0)
		{
			if (last == NULL) htab->table[ind] = e->next;
			else last->next = e->next;
			val = e->value;
			free(e);
			htab->counter--;
			return val;
		}
		last = e;
		e = e->next;
	}
	
	return -1;	// Not found !
}

int count_values (hashtable* htab)
{
	return htab->counter;
}

void destroy_table (hashtable* htab)
{
	uint i = 0;
	entry *e = NULL,
		  *last = NULL;
	
	for (i = 0; i < htab->length; i++)
	{
		e = htab->table[i];
		while (e != NULL)
		{
			last = e;
			e = e->next;
			free(last);
		}
	}
	free(htab->table);
	free(htab);
}

unsigned long hash (char* str)
{
	unsigned long h = 5381;
	uint c = 0;

	while((c = *str) != '\0') {
		h = ((h << 5) + h) + c;
		str++;
	}

	return h;
}

