/**
 * @file	uintll.h
 *
 * @author	Francois Clad
 * @date	2012-03-12
 * @version	1.1
 */

#ifndef _UINTLL_H_
#define _UINTLL_H_

#include <stdlib.h>
#include "base.h"



/**************************
 * Structures definitions *
 **************************/


typedef struct _uintll {
	unsigned int value;
	struct _uintll *next;
} uintll;



/************************
 * Functions prototypes *
 ************************/


/**
 * @brief	Add a new element to the list
 *
 * @note	NO DUPLICATE CHECKING - Please consider using addUintll function instead
 *
 * @param	value	Value to be added
 * @param	next	Pointer to the next element
 *
 * @return	Pointer to the newly created element
 */
uintll			*newUintll	(unsigned int, uintll *);


/**
 * @brief	Add a new value to the list if it does not already exist 
 *
 * @param	value	Value to be added
 * @param	el		Pointer to the list head
 *
 * @return	Pointer to the list head (which may have changed)
 */
uintll			*addUintll	(unsigned int value, uintll *el);


/**
 * @brief	Remove and de-allocate the first element of a list
 *
 * @param	head	Pointer to the element to be removed	
 *
 * @return	Pointer to the new list head
 */
uintll			*delHead	(uintll *);


/**
 * @brief	Get the size of a list
 *
 * @param	el		Pointer to the list head
 *
 * @return	Number of elements in the list
 */
unsigned int	sizeUintll	(uintll *);


/**
 * @brief	Recursively de-allocate a list
 *
 * @param	el		Pointer to the list head
 */
void			freeUintll	(uintll *);


/**
 * @brief	Recursively copy a list
 *
 * @param el		Pointer to the list head
 *
 * @return	Pointer to a copy of the list
 */
uintll			*copyUintll	(uintll *el);


/**
 * @brief	Find an element within a list
 *
 * @param el		Pointer to the list head
 * @param val		Value we are looking for
 *
 * @return	Pointer to the first matching element, or NULL if not found
 */
uintll	*searchUintll	(uintll *el, uint val);


/**
 * @brief	Print a list
 *
 * @param el		Pointer to the list head
 * @param file		File to print the list in (possibly stdout/stderr)
 */
void	printUintll		(uintll *el, FILE *file);


#endif // _UINTLL_H_
