/**
 * @file	topology.h
 *
 * @author	Francois Clad
 * @date	2012-06-21
 * @version	0.1
 */

#ifndef _TOPOLOGY_H_
#define _TOPOLOGY_H_

/* Standard C libraries */
#include <stdio.h>
#include <stdlib.h>
#include <regex.h>

#include "base.h"
#include "hashtable.h"
#include "uintll.h"

#define BUFFER_SIZE 4096 //1024

#ifdef DEBUG
#define PRINT_LABEL(ID)	fprintf(stderr, "Node %u: %s\n", ID, topo->nodes[ID]->label)
#else
#define PRINT_LABEL(...)
#endif /* DEBUG */


typedef struct _topo Topology;
typedef struct _node Node;
typedef struct _linkll Linkll;

struct _topo {
	uint	size;
	uint	max_size;
	Node	**nodes;
	uint	nb_edges;
	uint	nb_paths;
};

struct _node {
	char	*label;
	Linkll	*neighbors;		// List of direct neighbors
	uintll	**dests;		// Paths towards each destination
};

struct _linkll {
	uint	id;
	uint	dst;
	Linkll	*next;
};



Topology	*newTopology	(void);
void		freeTopology	(Topology *);

void		importGraph		(Topology *, hashtable *, char *);

void		printRoutes		(Topology *, uint, char *);

uint		get_hid			(hashtable *, Topology *, char *);


#endif // _TOPOLOGY_H_

