#!/usr/bin/python
# coding: utf-8
'''
Created on 2 sept. 2014

@author: slienardy
'''

import sys, os, argparse
import networkx as nx

from exportGraph import printGraph
from faketopo import MerlinParser


def merlinToNx(nodes, links):
    mp = MerlinParser()
    
    node_list, edge_list = mp.get_and_parse(nodes, links)
    
    G = nx.Graph()
    
    G.add_nodes_from(node_list)
    G.add_edges_from(edge_list)
    
    name = os.path.splitext(nodes)[0]
    
    return G, name

def gmlToNx(gml_file):
    
    name = os.path.splitext(gml_file)[0]
    
    return nx.read_gml(gml_file), name

def topoConvert(args):
    
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--output', help="Name of the output file", dest='output')
    
    # Input file(s) type are mutually exclusives but exactly one type is mandatory.
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-m', '--merlin', nargs=2, help="A pair [nodes links] of merlin files representing the topology", dest='merlin')
    group.add_argument('-g', '--gml', help="A GML file representing the topology", dest='gml')
    group.add_argument('-nx', '--networkx', help="A networkx graph dumped with pickle", dest='pickle')
    
    namespace = parser.parse_args(args)
    
    outfile = namespace.output + '.gv' if namespace.output is not None else ''
    
    if (namespace.pickle is not None):
        # I.e networkx dump -> already coded by F. Clad
        printGraph(namespace.pickle, outfile)
    else:
        H = nx.Graph()
        
        if (namespace.merlin is not None):
            G, out_name = merlinToNx(namespace.merlin[0], namespace.merlin[1])
        
        else :
            G, out_name = gmlToNx(namespace.gml)
        
        for e in G.edges():
            if e[0] != e[1]:
                H.add_edge(*e)
        
        print H.__dict__

        if outfile is '':
            outfile = out_name + '.gv'
            
        nx.write_dot(nx.connected_component_subgraphs(H)[0], outfile)
        

if __name__ == '__main__':
    
    print sys.argv
    topoConvert(sys.argv[1:])
    